mod util;
use core::{borrow::BorrowMut, ops::Deref};

use alloc::boxed::Box;
use lazy_static::lazy_static;
use multiboot2::{BootInformation, BootInformationHeader, FramebufferTag, FramebufferType, TagTrait};
use nalgebra::Vector2;
use spin::{Mutex, MutexGuard};
pub use util::is_video_mode_enabled;


#[derive(Clone, Copy)]
pub struct Color {
    r: u8,
    g: u8,
    b: u8,
}

impl Color {
    pub fn new(r: u8, g: u8, b: u8) -> Self {
        Self {r, g, b}
    }
}

pub struct VideoControler{
    component_size: u8,
    r_ofs: u8,
    g_ofs: u8,
    b_ofs: u8,
    buf_addr: u64,
    size: Vector2<usize>,
}

impl VideoControler {
    pub fn new(mb2p: *const BootInformationHeader) -> Self {
        let boot_info = unsafe { BootInformation::load(mb2p).unwrap() };
        let fbt = boot_info.framebuffer_tag().unwrap().unwrap();
        let FramebufferType::RGB { red, green, blue } = fbt.buffer_type().unwrap() else {
            panic!();
        };
        assert!(red.size == green.size && green.size == blue.size);
        log::info!("VGA VIDEO adddr: {:x}", fbt.address());
        log::info!("VGA VIDEO FB TYPE: {:?}", fbt.buffer_type());
        Self {
            component_size: red.size,
            r_ofs: red.position,
            g_ofs: green.position,
            b_ofs: blue.position,
            buf_addr: fbt.address(),
            size: Vector2::new(fbt.width() as usize, fbt.height() as usize)
        }
    }

    pub fn set_px(&self, x: usize, y: usize, c: Color) {
        let ptr = self.buf_addr as *mut () as *mut u8;
        let px = ptr.wrapping_add(x * self.size.x + y);
        unsafe {
            *px.wrapping_add((self.r_ofs % 8) as usize) = c.r;
            *px.wrapping_add((self.g_ofs % 8) as usize) = c.g;
            *px.wrapping_add((self.b_ofs % 8) as usize) = c.b;
        }
    }

    pub fn fill_screen(&self, c: Color) {    
        for x in 0..100 {
            for y in 0..100 {
                self.set_px(x, y, c);
            }
        }
    }
    
}
